#MenuTitle: Copy kerning from roman font to italic font
# -*- coding: utf-8 -*-
__doc__="""
Replaces kerning for all masters in the italic font. The italic font should be active when running this script.
"""
import GlyphsApp, math, copy


def log_error(msg):
  print("Error:", msg)
  Glyphs.showMacroWindow()


def calc_italic_axis_indices(font):
  italAxesIds = set(["ital", "slnt"])
  indices = set()
  for i in range(len(font.axes)):
    a = font.axes[i]
    if a.axisTag in italAxesIds:
      indices.add(i)
  return indices


def gen_vf_instance_key(axisValues, excludeAxisIndices=()):
  k = []
  for i in range(len(axisValues)):
    if excludeAxisIndices and i in excludeAxisIndices:
      continue
    k.append(str(axisValues[i]))
  return ",".join(k)


def build_masters_list(roman_font, ital_font):
  italAxisIndices = calc_italic_axis_indices(ital_font)

  # build vf key to roman master map
  romans = {}
  for roman_master in roman_font.masters:
    instanceKey = gen_vf_instance_key(roman_master.axes, italAxisIndices)
    romans[instanceKey] = roman_master

  print("master mapping:")
  # ital_master.id => roman_master
  masters = []
  for ital_master in ital_font.masters:
    instanceKey = gen_vf_instance_key(ital_master.axes, italAxisIndices)
    # crash here if there's roman<>ital master mismatch
    roman_master = romans[instanceKey]
    masters.append((roman_master, ital_master))
    print("  %-15s -> %s" % (roman_master.name, ital_master.name))
  return masters


def build_glyphs_id_map(master):
  glyphs = {}
  for g in master.glyphs:
    glyphs[g.id] = g
  return glyphs


def map_kerning_name(name, src_glyph_id_map, dst_font):
  if name[0] == "@":
    # group name
    return name
  # glyph ID
  glyph_name = src_glyph_id_map[name].name
  return dst_font.glyphs[glyph_name].id


def copy_master_kerning(src_master, src_glyphs, dst_master, dst_glyphs):
  #print("copy_master_kerning %r -> %r" % (src_master.name, dst_master.name))
  src_kerning = src_master.font.kerning[src_master.id]
  dst_kerning = {}

  for left_name, right_dict in src_kerning.items():
    left_name = map_kerning_name(left_name, src_glyphs, dst_master.font)

    dst_right_dict = {}
    for right_name, value in right_dict.items():
      right_name = map_kerning_name(right_name, src_glyphs, dst_master.font)
      dst_right_dict[right_name] = value
      #print("R", right_name, "->", value)

    dst_kerning[left_name] = dst_right_dict

  dst_master.font.kerning[dst_master.id] = dst_kerning


def main():
  Glyphs.clearLog()
  Glyphs.showMacroWindow()

  ital_font = Glyphs.font
  roman_font = None

  # make sure selected font is ital
  for m in ital_font.masters:
    if int(m.italicAngle) == 0:
      log_error("selected font is not italic or not all masters of the italic font has been loaded in the UI")
      return

  # find roman font
  for font in Glyphs.fonts:
    if font.familyName == ital_font.familyName:
      is_ital = False
      for m in font.masters:
        if int(m.italicAngle) != 0:
          is_ital = True
          break
      if not is_ital and roman_font is None:
        roman_font = font
        break

  if roman_font is None:
    log_error("no roman font found with family name \"%s\"" % ital_font.familyName)
    return

  print("roman_font", roman_font.filepath)
  print("ital_font ", ital_font.filepath)
  masters = build_masters_list(roman_font, ital_font)
  roman_glyphs = build_glyphs_id_map(roman_font)
  ital_glyphs = build_glyphs_id_map(ital_font)

  try:
    ital_font.disableUpdateInterface()
    for roman_master, ital_master in masters:
      copy_master_kerning(roman_master, roman_glyphs, ital_master, ital_glyphs)
    print("Done")

  finally:
    ital_font.enableUpdateInterface()

main()
